/*
* Copyright © 2021-2022. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.functions;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.NullValueException;
import com.streambase.sb.Schema;
import com.streambase.sb.Schema.Field;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Timestamp;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.client.CustomFunctionResolver;

public class TransformFunctions {

	/** Name tuple field */
	private static String NAME_FIELD = "name";
	
	/** Value tuple field */
	private static String VALUE_FIELD = "value";

	/** Format tuple field */
	private static String FORMAT_FIELD = "format";

	/** Canonical tuple field name */
	private static String CANONICAL_FIELD = "canonical";
	
	/** Mnemonic tuple field name */
	private static String MNEMONIC_FIELD = "mnemonic";
	

	/**
	 * Transforms a mnemonic name-value pair list into a tuple.
	 * @param definitions list of mapping definitions
	 * @param mnemonicValues list of name-value pairs
	 * @param output output object
	 * @return the output object
	 * @throws StreamBaseException streambase exception
	 * @throws ParseException parse exception
	 */
	@CustomFunctionResolver("transformDictToTupleResolver0")
	public static Tuple transformDictToTuple(ArrayList<Tuple> definitions, ArrayList<Tuple> mnemonicValues, Tuple output) throws StreamBaseException, ParseException {
		try {
			// Definitions schema:
			//   - datasource (PK) -- source name (by the time this is called, should be filtered to relevant rows anyways)
			//   - mnemonic (PK) -- source mnemonic name
			//   - canonical -- target canonical field
			//   - format -- format for parsing date/time fields
			
			// Mnemonics schema:
			//   - name (PK) -- mnemonic name
			//   - value -- mnemonic value
			
			
			// Validate that the output tuple is not null, otherwise won't be able to get the schema and process
			if(output == null) 
				throw new StreamBaseException("Output tuple must not be null");
			
			// Get the output schema
			Schema outputSchema = output.getSchema();
			
			// Build the definition map
			Map<String,List<Tuple>> definitionMap = buildDefinitionMap(definitions);
	
			// Loop over the input mnemonics
			for(Tuple thisMnemonicValue : mnemonicValues) {
				// Get the name and value
				String name = thisMnemonicValue.getString(NAME_FIELD);
				if(thisMnemonicValue.isNull(VALUE_FIELD))
					continue;
				String value = thisMnemonicValue.getString(VALUE_FIELD);
				
				processMnemonicValue(definitionMap, outputSchema, output, name, value);
			}
			
			// Clear the defintion map
			definitionMap = null;
			
			// Return the output
			return output;
		}
		catch(Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}

	public static CompleteDataType transformDictToTupleResolver0(CompleteDataType definitions,
			CompleteDataType mnemonics, CompleteDataType output) {
		return output;
	}

	/**
	 * Transforms a one tuple into another tuple.
	 * @param definitions list of mapping definitions
	 * @param input input object
	 * @param output output object
	 * @return the output object
	 * @throws StreamBaseException streambase exception
	 * @throws ParseException parse exception
	 */
	@CustomFunctionResolver("transformTupleToTupleResolver0")
	public static Tuple transformTupleToTuple(ArrayList<Tuple> definitions, Tuple input, Tuple output) throws StreamBaseException, ParseException {
		try {
			// Definitions schema:
			//   - datasource (PK) -- source name (by the time this is called, should be filtered to relevant rows anyways)
			//   - mnemonic (PK) -- source mnemonic name
			//   - canonical -- target canonical field
			//   - format -- format for parsing date/time fields
					
			
			// Validate that the output tuple is not null, otherwise won't be able to get the schema and process
			if(output == null) 
				throw new StreamBaseException("Output tuple must not be null");
			
			// Get the output schema
			Schema outputSchema = output.getSchema();
			
			// Build the definition map
			Map<String,List<Tuple>> definitionMap = buildDefinitionMap(definitions);
	
			// Loop over the input tuple fields
			Map<String, Object> inputFields = input.toMapView();
			Iterator<String> it = inputFields.keySet().iterator();
			while(it.hasNext()) {
				String name = it.next();
				Object value = inputFields.get(name);
			
				processMnemonicValue(definitionMap, outputSchema, output, name, value);
			}
					
			// Clear the defintion map
			definitionMap = null;
			
			// Return the output
			return output;
		}
		catch(Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}

	public static CompleteDataType transformTupleToTupleResolver0(CompleteDataType definitions,
			CompleteDataType input, CompleteDataType output) {
		return output;
	}


	private static Map<String,List<Tuple>> buildDefinitionMap(ArrayList<Tuple> definitions) throws NullValueException, TupleException {
		// Build index of definitions: mnemonic to list of target canonical fields
		Map<String,List<Tuple>> definitionMap = new HashMap<String,List<Tuple>>();
		for(int idx = 0; idx < definitions.size(); idx++) {
			Tuple definition = definitions.get(idx);
			String mnemonic = definition.getString(MNEMONIC_FIELD);
			List<Tuple> list = definitionMap.get(mnemonic);
			if(list == null) {
				list = new ArrayList<Tuple>();
				definitionMap.put(mnemonic, list);
			}
			list.add(definition);
		}
		
		return definitionMap;
	}

	private static void processMnemonicValue(Map<String,List<Tuple>> definitionMap, Schema outputSchema, Tuple output, String name, Object value) throws StreamBaseException, ParseException {
		// Retrieve the definition list using the name
		List<Tuple> definitionList = definitionMap.get(name);
		
		// If no definition found, skip this mnemonic
		if(definitionList == null || definitionList.size() == 0) 
			return;
		
		for(Tuple thisDefinition : definitionList) {
			// Get the canonical field from the definition
			String canonical = thisDefinition.getString(CANONICAL_FIELD);
			
			// Verify the output schema has the canonical field, if not, skip this mnemonic
			if(outputSchema.hasField(canonical)) {
				// Get the field
				Field field = outputSchema.getField(canonical);
				
				// Get the field data type
				// If timestamp format, need to parse the string into timestamp
				if(field.getCompleteDataType() == CompleteDataType.forTimestamp() && value != null && value.equals("null") == false) {
					output.setField(field, makeTimestamp(thisDefinition, name, value));
				}
				// Otherwise just set the value, other types should cast automatically
				else if(value != null && value.equals("null") == false){
					output.setField(field, value);
				}
			}
		}		
	}
	
	private static Object makeTimestamp(Tuple definition, String name, Object value) throws StreamBaseException, ParseException {
		// First check to see if the value is a number or a string
		if(isNumeric(value) == true) {
			// Convert the value to a double
			double d = Double.parseDouble(value.toString());
			
			// See if there is a format
			if(definition.isNull(FORMAT_FIELD) == false) {
				String format = definition.getString(FORMAT_FIELD);
				if(format.startsWith("x")) {
					double multiplier = Double.parseDouble(format.substring(1));
					d = d * multiplier;
				}
			}
			
			return Timestamp.msecs(Timestamp.TIMESTAMP, d);				
		}
		// Otherwise try to process as a string
		else if(value instanceof String && definition.isNull(FORMAT_FIELD) == false) {
			// Get the format
			String format = definition.getString(FORMAT_FIELD);
			
			SimpleDateFormat sdf = new SimpleDateFormat(format);
			java.util.Date date = sdf.parse(value.toString());
			Timestamp timestamp = new Timestamp(date);
			return timestamp;
		}
		// Otherwise just return the value
		else {
			return value;
		}
		
	}
	
	private static boolean isNumeric(Object value) {
		if(value instanceof Integer || value instanceof Long || value instanceof Double || value instanceof Float)
			return true;
		else if(value instanceof String) {
			try {
				Double.parseDouble((String) value);
				return true;
			}
			catch(NumberFormatException nfe) {
				return false;
			}
		}
		else {
			return false;
		}
	}
}
