/*
* Copyright © 2019-2022. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.operators.file;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.*;
import com.streambase.sb.operator.*;
import com.tibco.accelerators.operators.file.handler.FileDeleteHandler;

public class FileDeleteOperator extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1490868795723L;
	private Logger logger;
	
	// Properties
	private String displayName = "File Delete";
	private Schema[] outputSchemas = null;
	
	// Adapter properties
	private String filename;
	private boolean synchronousDelete;
	private boolean passthroughFields;
	
	// Local variables
	private int inputPorts = 1;
	private int outputPorts = 1;

	public FileDeleteOperator() {
		super();
		logger = getLogger();
		setPortHints(inputPorts, outputPorts);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setFilename("");
		setSynchronousDelete(true);
		setPassthroughFields(false);

	}

	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);
		setCaptureStrategy(CaptureTransformStrategy.NEST);

		// Get the input schema
		Schema inputSchema = getInputSchema(0);
		
		// Validate filename
		if(this.getFilename() == null || this.getFilename().length() == 0) {
			try {
				inputSchema.getField("filename");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename", "Filename not specified in Adapter Properties or input filename tuple");
			}		
		}
		
		// Define output schemas
		Schema dataSchema = new Schema(null,
				Schema.createField(DataType.STRING, "filename"),
				Schema.createField(DataType.STRING, "result"));					
		
		// Check for passthrough
		if(this.getPassthroughFields() == true) {
			try {
				Schema passthroughSchema = inputSchema.getField("passthrough").getSchema();
				List<Schema.Field> fields = new ArrayList<Schema.Field>(dataSchema.fields());
				fields.add(Schema.createTupleField("passthrough", passthroughSchema));
				dataSchema = new Schema(null, fields); 
			} catch (TupleException e) {
				e.printStackTrace();
			}
		}

		// Set data output schema
		setOutputSchema(0, dataSchema);
		
	}

	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		FileDeleteHandler operation = new FileDeleteHandler(logger, this, tuple);
		if(this.getSynchronousDelete() == true) {
			operation.run();
		}
		else {
			operation.start();
		}
	}

	public void init() throws StreamBaseException {
		super.init();
		// for best performance, consider caching input or output Schema.Field objects for
		// use later in processTuple()
		outputSchemas = new Schema[outputPorts];

		for (int i = 0; i < outputPorts; ++i) {
			outputSchemas[i] = getRuntimeOutputSchema(i);
		}
	}

	public Schema getCachedOutputSchema(int i) {
		return outputSchemas[i];
	}
	
	public void shutdown() {

	}

	@Override
	public URL getIconResource(IconKind iconType) {
		switch (iconType) {
		case CANVAS_OVERLAY_ICON:
			return getClass().getResource("icons/filedelete_48.png");
		case PALETTE_ICON_LARGE:
			return getClass().getResource("icons/filedelete_32.png");
		case PALETTE_ICON_SMALL:
			return getClass().getResource("icons/filedelete_16.png");
		}
		return null;
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	/**
	 * 
	 * @param filename the filename
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/**
	 * 
	 * @return the filename
	 */
	public String getFilename() {
		return this.filename;
	}

	/**
	 * 
	 * @param passthroughFields passthrouh fields
	 */
	public void setPassthroughFields(boolean passthroughFields) {
		this.passthroughFields = passthroughFields;
	}

	/**
	 * 
	 * @return passthrouh fields
	 */
	public boolean getPassthroughFields() {
		return this.passthroughFields;
	}

	/**
	 * 
	 * @param synchronousDelete synchronous delete
	 */
	public void setSynchronousDelete(boolean synchronousDelete) {
		this.synchronousDelete = synchronousDelete;
	}

	/**
	 * 
	 * @return synchronous delete
	 */
	public boolean getSynchronousDelete() {
		return this.synchronousDelete;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	/**
	 * 
	 * @return enable filename
	 */
	public boolean shouldEnableFilename() {
		return true;
	}

	/**
	 * 
	 * @return enable filename
	 */
	public boolean shouldEnableSynchronousDelete() {
		return true;
	}
	
	/**
	 * 
	 * @return enable passthrough fields
	 */
	public boolean shouldEnablePassthroughFields() {
		return true;
	}

}
