/*
* Copyright © 2019-2022. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.operators.file;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.*;
import com.streambase.sb.operator.*;
import com.tibco.accelerators.operators.file.handler.FileListHandler;

public class FileListOperator extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1490868795723L;
	private Logger logger;
	
	// Properties
	private String displayName = "File List";
	private Schema[] outputSchemas = null;
	
	// Adapter properties
	private String listPath;
	private boolean searchSubDirs;
	private String listFilter;
	private boolean oneTuplePerFile;
	private boolean synchronousList;
	private boolean passthroughFields;
	
	// Local variables
	private int inputPorts = 1;
	private int outputPorts = 1;

	public FileListOperator() {
		super();
		logger = getLogger();
		setPortHints(inputPorts, outputPorts);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		
		setListPath("");
		setSearchSubDirs(false);
		setListFilter("");
		setOneTuplePerFile(false);
		setSynchronousList(true);
		setPassthroughFields(false);

	}

	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);
		setCaptureStrategy(CaptureTransformStrategy.NEST);

		// Get the input schema
		Schema inputSchema = getInputSchema(0);
		
		// Validate list path
		if(this.getListPath() == null || this.getListPath().length() == 0) {
			try {
				inputSchema.getField("listPath");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename", "List path not specified in Adapter Properties or input tuple");
			}		
		}
		
		// Define output schemas
		Schema fileSchema = new Schema(null,
				Schema.createField(DataType.STRING, "absolutePath"),
				Schema.createField(DataType.STRING, "canonicalPath"),
				Schema.createField(DataType.STRING, "name"),
				Schema.createField(DataType.STRING, "parent"),
				Schema.createField(DataType.STRING, "path"),
				Schema.createField(DataType.TIMESTAMP, "lastModified"),
				Schema.createField(DataType.LONG, "length"),
				Schema.createField(DataType.STRING, "type"));					
		Schema dataSchema = null;
		if(this.getOneTuplePerFile() == true) {
			dataSchema = fileSchema;
		}
		else {
			dataSchema = new Schema(null,
				Schema.createListField("files", CompleteDataType.forTuple(fileSchema)));
		}
		
		
		// Check for passthrough
		if(this.getPassthroughFields() == true) {
			try {
				Schema passthroughSchema = inputSchema.getField("passthrough").getSchema();
				List<Schema.Field> fields = new ArrayList<Schema.Field>(dataSchema.fields());
				fields.add(Schema.createTupleField("passthrough", passthroughSchema));
				dataSchema = new Schema(null, fields); 
			} catch (TupleException e) {
				e.printStackTrace();
			}
		}

		// Set data output schema
		setOutputSchema(0, dataSchema);
		
	}

	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		FileListHandler operation = new FileListHandler(logger, this, tuple);
		if(this.getSynchronousList() == true) {
			operation.run();
		}
		else {
			operation.start();
		}
	}

	public void init() throws StreamBaseException {
		super.init();
		// for best performance, consider caching input or output Schema.Field objects for
		// use later in processTuple()
		outputSchemas = new Schema[outputPorts];

		for (int i = 0; i < outputPorts; ++i) {
			outputSchemas[i] = getRuntimeOutputSchema(i);
		}
	}

	public Schema getCachedOutputSchema(int i) {
		return outputSchemas[i];
	}
	
	public void shutdown() {

	}

	@Override
	public URL getIconResource(IconKind iconType) {
		switch (iconType) {
		case CANVAS_OVERLAY_ICON:
			return getClass().getResource("icons/filelist_48.png");
		case PALETTE_ICON_LARGE:
			return getClass().getResource("icons/filelist_32.png");
		case PALETTE_ICON_SMALL:
			return getClass().getResource("icons/filelist_16.png");
		}
		return null;
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	/**
	 * 
	 * @param listPath the list path
	 */
	public void setListPath(String listPath) {
		this.listPath = listPath;
	}

	/**
	 * 
	 * @return the list path
	 */
	public String getListPath() {
		return this.listPath;
	}

	/**
	 * 
	 * @param searchSubDirs search sub dirs
	 */
	public void setSearchSubDirs(boolean searchSubDirs) {
		this.searchSubDirs = searchSubDirs;
	}

	/**
	 * 
	 * @return search sub dirs
	 */
	public boolean getSearchSubDirs() {
		return this.searchSubDirs;
	}

	/**
	 * 
	 * @param listFilter the list filter
	 */	
	public void setListFilter(String listFilter) {
		this.listFilter = listFilter;
	}

	/**
	 * 
	 * @return the list filter
	 */
	public String getListFilter() {
		return this.listFilter;
	}

	/**
	 * 
	 * @param oneTuplePerFile one tuple per file
	 */
	public void setOneTuplePerFile(boolean oneTuplePerFile) {
		this.oneTuplePerFile = oneTuplePerFile;
	}

	/**
	 * 
	 * @return one tuple per file
	 */
	public boolean getOneTuplePerFile() {
		return this.oneTuplePerFile;
	}

	/**
	 * 
	 * @param synchronousList synchrnous list
	 */
	public void setSynchronousList(boolean synchronousList) {
		this.synchronousList = synchronousList;
	}

	/**
	 * 
	 * @return synchrnous list
	 */
	public boolean getSynchronousList() {
		return this.synchronousList;
	}

	/**
	 * 
	 * @param passthroughFields passthrough fields
	 */
	public void setPassthroughFields(boolean passthroughFields) {
		this.passthroughFields = passthroughFields;
	}

	/**
	 * 
	 * @return passthrough fields
	 */
	public boolean getPassthroughFields() {
		return this.passthroughFields;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	/**
	 * 
	 * @return enable list path
	 */
	public boolean shouldEnableListPath() {
		return true;
	}

	/**
	 * 
	 * @return enable search sub dirs
	 */
	public boolean shouldEnableSearchSubDirs() {
		return true;
	}
	
	/**
	 * 
	 * @return enable list filter
	 */
	public boolean shouldEnableListFilter() {
		return true;
	}

	/**
	 * 
	 * @return enable one tuple per file
	 */
	public boolean shouldEnableOneTuplePerFile() {
		return true;
	}
	
	/**
	 * 
	 * @return enable synchrnous list
	 */
	public boolean shouldEnableSynchronousList() {
		return true;
	}
	
	/**
	 * 
	 * @return enable passthrough fields
	 */
	public boolean shouldEnablePassthroughFields() {
		return true;
	}

}
