/*
* Copyright © 2019-2022. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.operators.file;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.*;
import com.streambase.sb.operator.*;
import com.tibco.accelerators.operators.file.handler.FileMoveHandler;

public class FileMoveOperator extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1490868795723L;
	private Logger logger;
	
	// Properties
	private String displayName = "File Move/Rename";
	private Schema[] outputSchemas = null;
	
	// Adapter properties
	private String sourceFilename;
	private String targetFilename;
	private boolean overwriteExisting;
	private boolean synchronousMove;
	private boolean passthroughFields;
	
	// Local variables
	private int inputPorts = 1;
	private int outputPorts = 1;

	public FileMoveOperator() {
		super();
		logger = getLogger();
		setPortHints(inputPorts, outputPorts);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setSourceFilename("");
		setTargetFilename("");
		setOverwriteExisting(true);
		setSynchronousMove(true);
		setPassthroughFields(false);

	}

	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);
		setCaptureStrategy(CaptureTransformStrategy.NEST);

		// Get the input schema
		Schema inputSchema = getInputSchema(0);
		
		// Validate source filename
		if(this.getSourceFilename() == null || this.getSourceFilename().length() == 0) {
			try {
				inputSchema.getField("sourceFilename");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename", "Source filename not specified in Adapter Properties or input filename tuple");
			}		
		}
		
		// Validate target filename
		if(this.getTargetFilename() == null || this.getTargetFilename().length() == 0) {
			try {
				inputSchema.getField("targetFilename");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename", "Source filename not specified in Adapter Properties or input filename tuple");
			}		
		}
		
		// Define output schemas
		Schema dataSchema = new Schema(null,
				Schema.createField(DataType.STRING, "sourceFilename"),
				Schema.createField(DataType.STRING, "targetFilename"),
				Schema.createField(DataType.STRING, "result"));					
		
		// Check for passthrough
		if(this.getPassthroughFields() == true) {
			try {
				Schema passthroughSchema = inputSchema.getField("passthrough").getSchema();
				List<Schema.Field> fields = new ArrayList<Schema.Field>(dataSchema.fields());
				fields.add(Schema.createTupleField("passthrough", passthroughSchema));
				dataSchema = new Schema(null, fields); 
			} catch (TupleException e) {
				e.printStackTrace();
			}
		}

		// Set data output schema
		setOutputSchema(0, dataSchema);
		
	}

	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		FileMoveHandler operation = new FileMoveHandler(logger, this, tuple);
		if(this.getSynchronousMove() == true) {
			operation.run();
		}
		else {
			operation.start();
		}
	}

	public void init() throws StreamBaseException {
		super.init();
		// for best performance, consider caching input or output Schema.Field objects for
		// use later in processTuple()
		outputSchemas = new Schema[outputPorts];

		for (int i = 0; i < outputPorts; ++i) {
			outputSchemas[i] = getRuntimeOutputSchema(i);
		}
	}

	public Schema getCachedOutputSchema(int i) {
		return outputSchemas[i];
	}
	
	public void shutdown() {

	}

	@Override
	public URL getIconResource(IconKind iconType) {
		switch (iconType) {
		case CANVAS_OVERLAY_ICON:
			return getClass().getResource("icons/filemove_48.png");
		case PALETTE_ICON_LARGE:
			return getClass().getResource("icons/filemove_32.png");
		case PALETTE_ICON_SMALL:
			return getClass().getResource("icons/filemove_16.png");
		}
		return null;
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	/**
	 * 
	 * @param overwriteExisting overwrite existing
	 */
	public void setOverwriteExisting(boolean overwriteExisting) {
		this.overwriteExisting = overwriteExisting;
	}

	/**
	 * 
	 * @return overwrite existing
	 */
	public boolean getOverwriteExisting() {
		return this.overwriteExisting;
	}

	/**
	 * 
	 * @param sourceFilename the source filename
	 */
	public void setSourceFilename(String sourceFilename) {
		this.sourceFilename = sourceFilename;
	}

	/**
	 * 
	 * @return the source filename
	 */
	public String getSourceFilename() {
		return this.sourceFilename;
	}

	/**
	 * 
	 * @param targetFilename the target filename
	 */
	public void setTargetFilename(String targetFilename) {
		this.targetFilename = targetFilename;
	}

	/**
	 * 
	 * @return the target filename
	 */
	public String getTargetFilename() {
		return this.targetFilename;
	}

	/**
	 * 
	 * @param passthroughFields passthrough fields
	 */
	public void setPassthroughFields(boolean passthroughFields) {
		this.passthroughFields = passthroughFields;
	}

	/**
	 * 
	 * @return passthrough fields
	 */
	public boolean getPassthroughFields() {
		return this.passthroughFields;
	}

	/**
	 * 
	 * @param synchronousMove synchronous move
	 */
	public void setSynchronousMove(boolean synchronousMove) {
		this.synchronousMove = synchronousMove;
	}

	/**
	 * 
	 * @return synchronous move
	 */
	public boolean getSynchronousMove() {
		return this.synchronousMove;
	}


	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	/**
	 * 
	 * @return enable source filename
	 */
	public boolean shouldEnableSourceFilename() {
		return true;
	}

	/**
	 * 
	 * @return enable target filename
	 */
	public boolean shouldEnableTargetFilename() {
		return true;
	}

	/**
	 * 
	 * @return enable synchronous move
	 */
	public boolean shouldEnableSynchronousMove() {
		return true;
	}
	
	/**
	 * 
	 * @return enable passthrough fields
	 */
	public boolean shouldEnablePassthroughFields() {
		return true;
	}

}
