/*
* Copyright © 2019-2022. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.operators.file.handler;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.ByteArrayView;
import com.streambase.sb.NullValueException;
import com.streambase.sb.Schema;
import com.streambase.sb.Schema.Field;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleCSVOutputWriter;
import com.streambase.sb.TupleException;
import com.streambase.sb.TupleJSONUtil;
import com.streambase.sb.util.CSV;
import com.streambase.sb.util.Util;
import com.tibco.accelerators.operators.file.FileWriterOperator;
import com.tibco.accelerators.operators.file.render.TupleToCsvRenderer;

public class FileWriterHandler extends Thread {
	private FileWriterOperator operator = null;
	private Logger logger = null;
	private Tuple inputTuple = null;
	private boolean createdFile = false;
	
	public FileWriterHandler(Logger logger, FileWriterOperator operator, Tuple tuple) {
		this.logger = logger;
		this.operator = operator;
		this.inputTuple = tuple;
	}

	public void run() {
		// Setup the filename
		String filename = this.getFilename();
		this.sendEventTuple("Write", "Start", filename, "Start processing");

		try {
			// Validate the file exists
			this.validateFileExists(filename);

			if(this.operator.getWriteAsBinary() == true) {
				this.writeFileBinary(filename, this.inputTuple.getBlobBuffer(0));
			}
			else {
				List<String> lines = null;			
				if(this.operator.getRenderAs().equals("None")) {
					lines = this.renderForNone();
				}
				else if(this.operator.getRenderAs().equals("Line")) {
					lines = this.renderForLines();
				}
				else if(this.operator.getRenderAs().equals("CSV")) {
					lines = this.renderForCsv();
				}
				else if(this.operator.getRenderAs().equals("JSON")) {
					lines = this.renderForJson();
				}
				
				// Send the file contents
				this.writeFileContents(filename, lines);
			}
			
			Tuple dataOutputTuple = this.operator.getOutputSchema(0).createTuple();
			dataOutputTuple.setString("filename", filename);
			this.sendDataTuple(dataOutputTuple);
			
			this.sendEventTuple("Write", "Finish", filename,
					"Completed processing");

		} catch (Exception ex) {
			//ex.printStackTrace();
			this.logger.error(ex.getMessage());
			this.sendEventTuple("Write", "Error", filename, ex.getMessage());
		}
	}

	private String getFilename() {
		// Setup the filename
		String filename = this.operator.getControlFilename();
		if(filename == null)
			filename = this.operator.getFilename();

		return filename;
	}

	private void validateFileExists(String filename) throws StreamBaseException, IOException {
		File f = new File(filename);
		if (filename == null || f.exists() == false) {
			if(this.operator.getFileDoesntExistAction() == FileDoesntExist.FAIL) {
				String message = "Error opening file '" + filename	+ "' not found";
				throw new StreamBaseException(message);
			}
			else if(this.operator.getFileDoesntExistAction() == FileDoesntExist.CREATE_NEW_FILE){
				f.createNewFile();
				this.createdFile = true;
			}
		}
		else {
			if(this.operator.getFileExistsAction() == FileExists.FAIL) {
				String message = "Error opening file '" + filename	+ "' already exists";
				throw new StreamBaseException(message);
			}
			else if(this.operator.getFileExistsAction() == FileExists.TRUNCATE_EXISTING) {
				f.delete();
				f.createNewFile();
				this.createdFile = true;
			}
		}
	}

	private void sendEventTuple(String type, String action, String object,
			String message) {
		if (this.operator.getEventPort() == true) {
			Schema eventOutputSchema = this.operator.getCachedOutputSchema(1);
			Tuple eventOutputTuple = eventOutputSchema.createTuple();
			try {
				eventOutputTuple.setString("type", type);
				eventOutputTuple.setString("action", action);
				eventOutputTuple.setString("object", object);
				eventOutputTuple.setString("message", message);

				this.operator.sendOutput(1, eventOutputTuple);
			} catch (StreamBaseException se) {
				se.printStackTrace();
			}
		}
	}

	private void sendDataTuple(Tuple dataOutputTuple)
			throws StreamBaseException {
		// Append passthrough
		this.appendPassthrough(dataOutputTuple);

		// Send data tuple
		this.operator.sendOutput(0, dataOutputTuple);
	}

	private List<String> renderForNone() throws NullValueException, TupleException {
		List<String> lines = new ArrayList<String>();
		String s = this.inputTuple.getString(0);
		lines.add(s);
		return lines;
	}
	
	private List<String> renderForLines() throws NullValueException, TupleException {
		List<String> lines = new ArrayList<String>();
		
		if(this.operator.getOneLinePerTuple() == true) {
			String s = this.inputTuple.getString(0);
			lines.add(s);
		}
		else {
			List<?> lineList = this.inputTuple.getList(this.operator.getListTupleName());
			for(int idx = 0; idx < lineList.size(); idx++) {
				lines.add((String) lineList.get(idx));
			}
		}
		
		return lines;
	}
	
	private List<String> renderForCsv() throws NullValueException, TupleException, CloneNotSupportedException {		
		TupleToCsvRenderer renderer = new TupleToCsvRenderer();
		List<String> lines = new ArrayList<String>();

		if(this.operator.getIncludeHeader() == true && this.createdFile == true) {
			String header = null;
			if(this.operator.getOneLinePerTuple() == true) {
				header = renderer.renderTupleToHeader(this.inputTuple, this.operator.getPassthroughFields(), 
						this.operator.getFieldDelimiter());
			}
			else {
				List<?> lineList = this.inputTuple.getList(this.operator.getListTupleName());
				if(lineList.size() > 0) {
					Tuple lineTuple = (Tuple) lineList.get(0);
					header = renderer.renderTupleToHeader(lineTuple, this.operator.getPassthroughFields(), 
							this.operator.getFieldDelimiter());
				}
			}
			if(header != null)
				lines.add(header);
		}

		Charset charset;
		if(this.operator.getUseDefaultCharset() == true) {
			charset = Charset.defaultCharset();
		}
		else {
			charset = Charset.availableCharsets().get(
					this.operator.getCharacterSet());

		}
		
		if(this.operator.getOneLinePerTuple() == true) {
			Tuple tuple = this.operator.getInputObjectSchema().createTuple();
			for(int idx = 0; idx < this.operator.getInputObjectSchema().getFieldCount(); idx++) {
				Field field = this.operator.getInputObjectSchema().getField(idx);
				tuple.setField(field, this.inputTuple.getField(field.getName()));
			}
			String line = this.tupleToCSV(tuple, charset);
 
			if(line != null)
				lines.add(line);			
		}
		else {
			List<?> lineList = this.inputTuple.getList(this.operator.getListTupleName());
			for(int idxTuple = 0; idxTuple < lineList.size(); idxTuple++) {
				Tuple thisTuple = (Tuple) lineList.get(idxTuple);
				/*Tuple tuple = this.operator.getInputObjectSchema().createTuple();
				for(int idx = 0; idx < this.operator.getInputObjectSchema().getFieldCount(); idx++) {
					Field field = this.operator.getInputObjectSchema().getField(idx);
					tuple.setField(field, thisTuple.getField(field.getName()));
				}*/
				String line = this.tupleToCSV(thisTuple, charset);
				//String line = renderer.renderTupleToLine(tuple, this.operator.getPassthroughFields(), this.operator.getFieldDelimiter(), 
				//		this.operator.getTimestampFormat(), this.operator.getNullAsEmpty(), this.operator.getNullValue());
				if(line != null)
					lines.add(line);			
			}
		}
		return lines;
	}
	
	private String tupleToCSV(Tuple tuple, Charset charset) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
        	char quoteChar = CSV.DEFAULT_QUOTE;
        	char delimChar = CSV.DEFAULT_DELIM;
        	if(this.operator.getFieldDelimiter() != null)
        		delimChar = this.operator.getFieldDelimiter().charAt(0);
        	String nullString = null;
        	if(this.operator.getNullAsEmpty() == true)
        		nullString = "";
        	else if(this.operator.getNullValue() != null)
        		nullString = this.operator.getNullValue();
        	DateFormat dateFormat = null;
        	if(this.operator.getTimestampFormat() != null)
        		dateFormat = new SimpleDateFormat(this.operator.getTimestampFormat());

        	String lineSeparator = null;
        	if(this.operator.getLineSeparator() != null && this.operator.getLineSeparator().length() > 0)
        		lineSeparator = this.operator.getLineSeparator();
        	
			//this.operator.getFieldDelimiter() == null ? CSV.DEFAULT_DELIM : this.operator.getFieldDelimiter().toCharArray()[0], 
			//		this.operator.getNullAsEmpty() == true ? "" : this.operator.getNullValue(), null);        	
        	CSV.FormatInfo csvFormat = new CSV.FormatInfo(quoteChar, delimChar, nullString, dateFormat);
            TupleCSVOutputWriter csvWriter = new TupleCSVOutputWriter(out, new com.streambase.sb.TupleCSVOutputWriter.FormatInfo(
            		csvFormat, false, lineSeparator, true, charset.name()));
            csvWriter.write(tuple);
            Util.close(csvWriter);
        } catch (IOException e) {
            assert false;
        }
        try {
        	String output = out.toString(charset.name()).trim();
        	return output;
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }

		//return CSV.tupleToCSV(tuple, charset);
		
		//String line = renderer.renderTupleToLine(tuple, this.operator.getPassthroughFields(), this.operator.getFieldDelimiter(), 
		//		this.operator.getTimestampFormat(), this.operator.getNullAsEmpty(), this.operator.getNullValue());
	}
	
	private List<String> renderForJson() throws NullValueException, TupleException, CloneNotSupportedException {		
		List<String> lines = new ArrayList<String>();

		Tuple tuple = this.operator.getInputObjectSchema().createTuple();
		for(int idx = 0; idx < this.operator.getInputObjectSchema().getFieldCount(); idx++) {
			Field field = this.operator.getInputObjectSchema().getField(idx);
			tuple.setField(field, this.inputTuple.getField(field.getName()));
		}
			
		EnumSet<TupleJSONUtil.Options> options = EnumSet.of(TupleJSONUtil.Options.PRETTY_FORMAT);		
		String line = TupleJSONUtil.toJSONString(tuple, options).toString();
		lines.add(line);
		return lines;
	}
	
	private void appendPassthrough(Tuple dataOutputTuple) throws TupleException {
		if (this.operator.getPassthroughFields() == true) {
			dataOutputTuple.setField("passthrough",
					this.inputTuple.getField("passthrough"));
		}
	}
	
	private void writeFileContents(String filename, List<String> lines) throws IOException {
		String encoding = this.operator.getCharacterSet();
		if (encoding == null || encoding.length() == 0)
			encoding = Charset.defaultCharset().name();

		File f = new File(filename);
		boolean append = this.operator.getFileExistsAction() == FileExists.APPEND_TO_EXISTING;
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f, append), encoding));
		for(int idx = 0; idx < lines.size(); idx++) {
			writer.write(lines.get(idx));
			writer.newLine();
		}
		writer.close();
	}
		
	private void writeFileBinary(String filename, ByteArrayView byteArrayView) throws IOException {
		File f = new File(filename);
		OutputStream os = new FileOutputStream(f);
		os.write(byteArrayView.array());
		os.close();
	}
}
