/*
* Copyright © 2019-2022. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.operators.file.parse;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.DataType;
import com.streambase.sb.Schema;
import com.streambase.sb.Timestamp;
import com.streambase.sb.Tuple;

public class CsvToTupleParser {
	private Logger logger = null;

	public CsvToTupleParser(Logger logger) {
		this.logger = logger;
	}

	public LineParseResult parseStringToLines(String separator,
			ParseHeader headerType, String data) {
		// Line separator
		String lineSeparator = separator;
		if (lineSeparator == null || lineSeparator.length() == 0)
			lineSeparator = "\n";
		List<String> tokens = new ArrayList<String>();
		String header = null;
		String[] tokenArr = data.split("\\" + lineSeparator);
		if (tokenArr == null)
			this.logger.info("IS NULL");
		for (int idx = 0; idx < tokenArr.length; idx++) {
			String token = tokenArr[idx];
			if (headerType == ParseHeader.IGNORE_HEADER && idx == 0)
				continue;
			else if (headerType == ParseHeader.READ_HEADER && idx == 0) {
				header = token;
				continue;
			}
			tokens.add(token);			
		}

		LineParseResult result = new LineParseResult(header, tokens);
		return result;
	}

	public Tuple parseLineToTuple(Schema schema, String separator,
			boolean emptyAsNull, String nullValue, String timestampFormat,
			boolean preserveWhitespace,
			ParseIncompleteRecords incompleteRecords, String header, String line) {
		// Field separator
		String fieldSeparator = separator;
		if (fieldSeparator == null || fieldSeparator.length() == 0)
			fieldSeparator = ",";

		try {
			Tuple tuple = schema.createTuple();
			List<String> tokens = this.parseLineToTokens(fieldSeparator, line);
			int index = 0;
			for (int idx = 0; idx < schema.getFieldCount(); idx++) {
				Schema.Field f = schema.getField(idx);
				String token = tokens.get(idx);
				if (preserveWhitespace == false)
					token = token.trim();
				if (token.length() == 0 && emptyAsNull == true)
					token = null;
				if (emptyAsNull == false && token.equals(nullValue))
					token = null;
				if (token == null)
					tuple.setNull(f);
				else {
					if (f.getDataType() == DataType.TIMESTAMP && token != null) {
						Timestamp tstamp = this.parseTimestamp(timestampFormat,
								token);
						tuple.setField(f, tstamp);
					} else {
						tuple.setField(f, token);
					}
				}
				index = index + 1;
			}

			// If the tuple is incomplete
			if (index < schema.getFieldCount()) {
				// If discard, just return null
				if (incompleteRecords == ParseIncompleteRecords.DISCARD) {
					this.logger.warn("Discarding incomplete record " + line);
					return null;
				}
				// Otherwise fill out with nulls
				else if (incompleteRecords == ParseIncompleteRecords.POPULATE_WITH_NULLS) {
					for (int idx = index; idx < schema.getFieldCount(); idx++) {
						Schema.Field f = schema.getField(idx);
						tuple.setNull(f);
					}
				}
			}

			return tuple;
		} catch (Exception ex) {
			ex.printStackTrace();
			this.logger.warn("Discarding record " + line + ": "
					+ ex.getMessage());
			return null;
		}
	}

	private List<String> parseLineToTokens(String fieldSeparator, String line) {
		List<String> tokensList = new ArrayList<String>();
		boolean inQuotes = false;
		StringBuilder b = new StringBuilder();
		char sep = fieldSeparator.charAt(0);
		for (char c : line.toCharArray()) {
			if(c == sep) {
				if (inQuotes) {
					b.append(c);
				} else {
					tokensList.add(b.toString());
					b = new StringBuilder();
				}
			}
			else if(c == '\"') {
				inQuotes = !inQuotes;
				//b.append(c);
			}
			else {
				b.append(c);
			}
		}
		tokensList.add(b.toString());

		return tokensList;
	}

	private Timestamp parseTimestamp(String timestampFormat, String token)
			throws ParseException {
		SimpleDateFormat sdf = new SimpleDateFormat(timestampFormat);
		java.util.Date d = sdf.parse(token);
		Timestamp tstamp = new Timestamp(d);
		return tstamp;
	}

	public class LineParseResult {
		private String header = null;
		private List<String> lines = null;

		public LineParseResult(String header, List<String> lines) {
			this.header = header;
			this.lines = lines;
		}

		public String getHeader() {
			return this.header;
		}

		public List<String> getLines() {
			return this.lines;
		}
	}
}
